// MIT License
//
// Copyright (c) 2020 Alex Howarth
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package tilt

import (
	"encoding/binary"
	"encoding/hex"
	"errors"
)

//iBeaconType identifies an iBeacon
const iBeaconType uint32 = 0x4c000215

// ErrNotBeacon - the BLE device is not an iBeacon
var ErrNotBeacon = errors.New("not an iBeacon")

// IBeacon data
type IBeacon struct {
	UUID  string
	Major uint16
	Minor uint16
}

// IsIBeacon to determine if data is an iBeacon
func IsIBeacon(data []byte) bool {
	if len(data) < 25 {
		return false
	}

	if binary.BigEndian.Uint32(data) != iBeaconType {
		return false
	}

	return true
}

// NewIBeacon creates an iBeacon from a valid BLE event
func NewIBeacon(data []byte) (b *IBeacon, err error) {
	if !IsIBeacon(data) {
		err = ErrNotBeacon
		return
	}

	b = &IBeacon{
		UUID:  hex.EncodeToString(data[4:20]),
		Major: binary.BigEndian.Uint16(data[20:22]),
		Minor: binary.BigEndian.Uint16(data[22:24]),
	}

	return
}
